@@ -9,7 +9,7 @@ import numpy as np
 
 import qtawesome as qta
 LOG_LEVEL = GLOBAL_LOG_LEVEL
-LOG_LEVEL = 10
+#LOG_LEVEL = 10
 
 class ModelUpdateController(Logger):
     _model = None
@@ -48,10 +48,6 @@ class ModelUpdateController(Logger):
     
     def disconnect_model(self):
         self.model.item_selector.disconnect(self)
-
-    
-    
-
         
 class EditPixelSizeController(ModelUpdateController):
     _geometry = None
@@ -812,7 +808,7 @@ class EditWallsController(EditModelItemControllerBase):
         item = self.get_item_in_view()
         
         if item is None:
-            print('No item selected!')
+            self.logger.debug('No item selected!')
             current_shielding = None
         else:
             current_shielding = item.shielding
