@@ -5,13 +5,14 @@ from PyQt5.QtWidgets import (QWidget, QCheckBox, QPushButton, QVBoxLayout,
 from PyQt5 import QtCore, QtGui
 import qtawesome as qta
 from pyrateshield import labels, Observable, GLOBAL_LOG_LEVEL
-from pyrateshield.constants import Constants
+from pyrateshield.gui import io
+from pyrateshield.constants import CONSTANTS
 
 MAX_VALUE = float('inf')
 MAX_INT_VALUE = 99999999
-CONSTANTS = Constants()
 
-WIDGET_WIDTH = 300
+
+
 LOG_LEVEL = GLOBAL_LOG_LEVEL
 #LOG_LEVEL = 10
 
@@ -79,6 +80,7 @@ class QDoubleSpinBoxInfinite(QDoubleSpinBox):
 class EditViewBase(QWidget, Observable):
     EVENT_VIEW_UPDATE = 'event_view_update'
     _layout = None
+    #LABEL = 'label'
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
         Observable.__init__(self, log_level=LOG_LEVEL)
@@ -107,9 +109,11 @@ class EditViewBase(QWidget, Observable):
             
         self.stretch_layout.addLayout(self.layout)
         if hasattr(self, 'explanation'):
-            label = QLabel(self.explanation)
-            label.setWordWrap(True)
-            self.stretch_layout.addWidget(label)
+            self.help_button = QPushButton('Help')
+            callback = lambda : io.show_help(title=self.LABEL,
+                                             text = self.explanation)
+            self.help_button.clicked.connect(callback)        
+            self.stretch_layout.addWidget(self.help_button)
             
         self.stretch_layout.addStretch(1)
         self.setLayout(self.stretch_layout)
@@ -134,7 +138,7 @@ class EditListViewBase(EditViewBase):
     def __init__(self, parent=None):
     
         super().__init__(parent)
-        self.setFixedWidth(WIDGET_WIDTH)
+        
         
     def create_widgets(self):
         self.list = QComboBox(self)
@@ -207,7 +211,7 @@ class EditListViewBase(EditViewBase):
         #self.disable_connection(event_name=self.EVENT_VIEW_UPDATE)
         for label, value in dct.items():
             if label == labels.NAME:
-                safe_set_value_to_widget(self.name_input, value)
+                safe_set_value_to_widget(self.name_input, value)   
                 self.logger.debug(f'Setting list to {value}')
                 safe_set_value_to_widget(self.list, value)
             elif label == labels.ENABLED:
@@ -286,6 +290,7 @@ class EditListViewPositionBase(EditListViewBase):
         
 
 class EditMaterialsView(EditListViewBase):
+    LABEL = labels.MATERIALS
     explanation =\
         ('Materials can be changed or added to a limited extend. '
          'Radtracer implements a limited set of materials. Pyshield uses '
@@ -424,6 +429,7 @@ class EditMaterialsView(EditListViewBase):
     
         
 class EditShieldingView(EditListViewBase):
+    LABEL = labels.SHIELDINGS
     _color = 'r'
     _DEFAULT_THICKNESS = 1
     _DEFAULT_LINEWIDTH = 1
@@ -558,9 +564,9 @@ class EditShieldingView(EditListViewBase):
         callback = lambda _, label=label: self.emit_change(label)
         self.line_width.valueChanged.connect(callback)
         
-       
         self.color_button.clicked.connect(self.select_color)
         
+        
     def create_layout_body(self):
         
         row = self.layout.rowCount() + 1
@@ -598,6 +604,8 @@ class EditShieldingView(EditListViewBase):
         self.line_width.setValue(self._DEFAULT_LINEWIDTH)
         
 class EditWallsView(EditViewBase):
+    LABEL = labels.WALLS
+    
     explanation = ("To add a Wall select Wall from the toolbar. "
                    "Click and hold the left mouse button to draw a wall")
     EVENT_SCROLL = 'event_scroll'
@@ -739,6 +747,7 @@ class EditWallsView(EditViewBase):
         self.shielding_list.setCurrentIndex(0)
         
 class EditClearanceView(EditListViewBase):
+    LABEL = labels.CLEARANCE
     explanation = ("\nUp to two biological fraction can be defined with a"
                    " corresponding biological halflife. For each fraction "
                    "pyrateshield will calculate an effective halflife by "
@@ -962,46 +971,10 @@ class EditClearanceView(EditListViewBase):
         #self.validate_fraction2()
         self.emit_change(label)
         self.enable_fraction2()
-        
-    # def set_decay_value1(self, _=None):
-    #    if self.decay_fraction2_checkbox.isChecked():
-    #        self.validate_fraction1()
-       
-    #    label = labels.DECAY_FRACTION1
-    #    self.emit_change(label)
-   
-    # def set_decay_value2(self, _=None):
-    #    if self.decay_fraction1_checkbox.isChecked():
-    #        self.validate_fraction2()
-       
-    #    label = labels.DECAY_FRACTION2
-    #    self.emit_change(label)
-       
-    # def validate_fraction1(self):
-    #    if self.split_time_checkbox.isChecked():
-    #        return
-    #    if self.decay_fraction1_checkbox.isChecked()\
-    #        and self.decay_fraction2_checkbox.isChecked():
-    #            f1 = self.decay_fraction1.value()
-    #            f2 = self.decay_fraction2.value()
-       
-    #            if (1-f1-f2) < 0:
-                   
-    #                self.decay_fraction1.setValue(1-f2)
-   
-    # def validate_fraction2(self):
-    #    if self.split_time_checkbox.isChecked():
-    #         return
-    #    if self.decay_fraction1_checkbox.isChecked()\
-    #        and self.decay_fraction2_checkbox.isChecked():
-    #            f1 = self.decay_fraction1.value()
-    #            f2 = self.decay_fraction2.value()
-       
-    #            if (1-f1-f2) < 0:
-    #                self.decay_fraction2.setValue(1-f1)
-            
+ 
         
 class EditSourcesNMView(EditListViewPositionBase):
+    LABEL = labels.SOURCES_NM
     explanation = ("To add a new source select Source NM from the toolbar and "
                    "click on the floorplan to add a source at that position."
                    "\n\n"
@@ -1011,7 +984,11 @@ class EditSourcesNMView(EditListViewPositionBase):
                    "spectrum after additional 10 cm of "
                    "shielding with water. Note that large differences in results "
                    "may occur between pyshield and radtracer when setting self "
-                   "shielding to 'Body'.")
+                   "shielding to 'Body.'\n\n"
+                   "Note that setting the self shielding to 'Body' may increase "
+                   "the dose rates for pyshield for some isotopes. This is due "
+                   "to the buildup in 10 cm of water. Using a user defined fixed "
+                   "factor could provide a better results for pyshield.")
                    
     
     def from_dict(self, dct):
@@ -1024,6 +1001,8 @@ class EditSourcesNMView(EditListViewPositionBase):
                 safe_set_value_to_widget(self.activity, value)
             elif label == labels.ISOTOPE:
                 safe_set_value_to_widget(self.isotope_input, value)
+                halflife = CONSTANTS.get_isotope_by_name(value).half_life
+                safe_set_value_to_widget(self.isotope_half_life_value, halflife)
             elif label == labels.SELF_SHIELDING:
                 if isinstance(value, str):
                     safe_set_value_to_widget(self.self_shielding_list, value)
@@ -1034,10 +1013,14 @@ class EditSourcesNMView(EditListViewPositionBase):
                 safe_set_value_to_widget(self.number_of_exams_input, value)
             elif label == labels.APPLY_DECAY_CORRECTION:
                 safe_set_value_to_widget(self.decay_correction, value)
-                half_life = CONSTANTS.get_isotope_by_name(dct[labels.ISOTOPE]).half_life
-                safe_set_value_to_widget(self.isotope_half_life_value, half_life)
+                self.toggle_halflife_visible()
             elif label == labels.CLEARANCE:
                 safe_set_value_to_widget(self.clearance_list, value)
+                
+    def toggle_halflife_visible(self):
+        visible = self.decay_correction.isChecked()
+        self.isotope_half_life_label.setEnabled(visible)
+        self.isotope_half_life_value.setEnabled(visible)
         
     def set_enabled(self, enabled):
         super().set_enabled(enabled)
@@ -1046,10 +1029,17 @@ class EditSourcesNMView(EditListViewPositionBase):
         if self.self_shielding_list.currentText() != labels.SELF_SHIELDING_FACTOR:
             enabled = False
         
-        self.logger.info(f'Setting Visble to {enabled}')
+        if enabled:
+            self.toggle_halflife_visible()
+       
+        
+        self.logger.debug(f'Setting Visble to {enabled}')
+        
         
         self.set_enabled_self_shielding(enabled)
         
+         
+        
     def set_enabled_self_shielding(self, enabled):
         self.self_shielding_factor_input.setEnabled(enabled)
         self.self_shielding_factor_input.setVisible(enabled)
@@ -1180,11 +1170,11 @@ class EditSourcesNMView(EditListViewPositionBase):
         row = self.layout.rowCount() + 1
         
         self.layout.addWidget(self.number_of_exams_label, row, 0)
-        self.layout.addWidget(self.number_of_exams_input, row, 1, 1, 2)
+        self.layout.addWidget(self.number_of_exams_input, row, 1)
         
         row += 1
         self.layout.addWidget(self.activity_label, row, 0)
-        self.layout.addWidget(self.activity, row, 1, 1, 2)
+        self.layout.addWidget(self.activity, row, 1)
         row += 1
         self.layout.addWidget(self.isotope_label, row, 0)
         self.layout.addWidget(self.isotope_input, row, 1)
@@ -1213,6 +1203,7 @@ class EditSourcesNMView(EditListViewPositionBase):
 
 
 class EditCriticalPointsView(EditListViewPositionBase):
+    LABEL = labels.CRITICAL_POINTS
     explanation = ("To add a new critcal point select Critical Point from "
                    "the toolbar and click on the floorplan to add a "
                    "critical point at that position.")
@@ -1265,6 +1256,7 @@ class EditCriticalPointsView(EditListViewPositionBase):
 
         
 class EditSourceXrayView(EditListViewPositionBase):
+    LABEL = labels.SOURCES_XRAY
     explanation = ("To add a new source select Source Xray from the toolbar and "
                    "click on the floorplan to add a source at that position.")
                    
@@ -1276,7 +1268,7 @@ class EditSourceXrayView(EditListViewPositionBase):
             if label == labels.DAP:
                 safe_set_value_to_widget(self.dap, value)
             elif label == labels.KVP:
-                safe_set_value_to_widget(self.kvp, value)
+                safe_set_value_to_widget(self.kvp, str(value))
             elif label == labels.NUMBER_OF_EXAMS:
                 safe_set_value_to_widget(self.number_of_exams_input, value)
         #self.enable_connection(event_name=self.EVENT_VIEW_UPDATE)
@@ -1342,6 +1334,7 @@ class EditSourceXrayView(EditListViewPositionBase):
         self.dap.clear()
 
 class EditSourceCTView(EditListViewPositionBase):
+    LABEL = labels.SOURCES_CT
     explanation = ("To add a new source select Source CT from the toolbar and "
                    "click on the floorplan to add a source at that position.")
     def from_dict(self, dct):
@@ -1351,7 +1344,7 @@ class EditSourceCTView(EditListViewPositionBase):
            if label == labels.DLP:
                safe_set_value_to_widget(self.dlp, value)
            elif label == labels.KVP:
-               safe_set_value_to_widget(self.kvp, value)
+               safe_set_value_to_widget(self.kvp, str(value))
            elif label == labels.NUMBER_OF_EXAMS:
                safe_set_value_to_widget(self.number_of_exams_input, value)
            elif label == labels.CT_BODY_PART:
@@ -1363,8 +1356,8 @@ class EditSourceCTView(EditListViewPositionBase):
     
     def to_dict(self):
         dct = super().to_dict()
-        dct[labels.NUMBER_OF_EXAMS] = safe_to_float(self.number_of_exams_input.value())
-        dct[labels.KVP] = safe_to_float(self.kvp.currentText())
+        dct[labels.NUMBER_OF_EXAMS] = safe_to_int(self.number_of_exams_input.value())
+        dct[labels.KVP] = safe_to_int(self.kvp.currentText())
         dct[labels.CT_BODY_PART] = self.body_part.currentText()
         dct[labels.DLP] = safe_to_float(self.dlp.value())
         return dct        
@@ -1398,13 +1391,19 @@ class EditSourceCTView(EditListViewPositionBase):
         callback = lambda _, label=label: self.emit_change(label)                
         self.number_of_exams_input.valueChanged.connect(callback)
         
-        label = labels.NUMBER_OF_EXAMS        
+        label = labels.KVP
         callback = lambda _, label=label: self.emit_change(label)        
         self.kvp.currentTextChanged.connect(callback)
+ 
         
         label = labels.DLP
         callback = lambda _, label=label: self.emit_change(label)        
         self.dlp.valueChanged.connect(callback)
+                
+        label = labels.CT_BODY_PART
+        callback = lambda _, label=label: self.emit_change(label)        
+        self.dlp.valueChanged.connect(callback)
+        self.body_part.currentTextChanged.connect(callback)
         
     def create_layout_body(self):
         row = self.layout.rowCount() + 1
@@ -1431,9 +1430,10 @@ class EditSourceCTView(EditListViewPositionBase):
 
  
 class EditPixelSizeView(EditViewBase):
+    LABEL = labels.PIXEL_SIZE_CM
     EVENT_GEOMETRY_RADIO_BUTTON = 'event_geometry_radio_button'
     explanation =\
-("\n\nThe pixel size can be set by hand or by a measurement on the floor plan " 
+("The pixel size can be set by hand or by a measurement on the floor plan " 
  "image. Measurement is done by drawing a line between two points for which "
  "the real world distance in cm is known.")
 
@@ -1553,5 +1553,5 @@ class EditPixelSizeView(EditViewBase):
 
         
         self.set_choose_measured()
-        self.setFixedWidth(WIDGET_WIDTH)
+       
         
