@@ -273,14 +273,14 @@ class Model(Observable):
             
             for wall in self.walls:
                 if wall.shielding == item.name:
-
-                    event_data = (labels.COLOR, item.color, item.color)
+                    event_data = (wall, labels.COLOR, item.color, item.color)
                     wall.emit(wall.EVENT_UPDATE, event_data)
                     
         elif label == labels.LINEWIDTH:
+         
             for wall in self.walls:
                 if wall.shielding == item.name:
-                    event_data = (labels.LINEWIDTH, item.linewidth, item.linewidth)
+                    event_data = (wall, labels.LINEWIDTH, item.linewidth, item.linewidth)
                     wall.emit(wall.EVENT_UPDATE, event_data)
                     
     def update_clearance(self, event_data):
